import { Injectable } from '@angular/core';
import { Config } from '../config';
import { Observable } from "rxjs/Observable";
import { Payload } from '../payload';
import { Events } from 'ionic-angular';
import { Socket } from '../socket';
import { Status } from '../status';
var Api = (function () {
    function Api(appConfig, socketService, payloadProvider, events) {
        this.appConfig = appConfig;
        this.socketService = socketService;
        this.payloadProvider = payloadProvider;
        this.events = events;
        if (!Api.transactions) {
            Api.transactions = {};
        }
        this.events.subscribe('socket:client:rx', this.transactionResponse);
    }
    Api.prototype.addTransaction = function (lambda, request, opts) {
        var _this = this;
        if (opts === void 0) { opts = {}; }
        return Observable.create(function (observer) {
            _this.payloadProvider.getPayload(request).then(function (payload) {
                var timeout = opts['timeout'] || _this.appConfig.get('api.timeout', 10000);
                var queue = opts['queue'] || false;
                var pid = payload['pid'];
                if (!queue && !_this.socketService.connected()) {
                    observer.error({ progress: 0, status: { code: Status.NOT_CONNECTED, message: Status.get(Status.NOT_CONNECTED) }, data: {} });
                    return;
                }
                console.log("ADD TRANSACTION FOR " + lambda + ": " + pid);
                Api.transactions[pid] = {
                    timeout: timeout,
                    latency: {
                        deviceSubmitAt: new Date().getTime(),
                    },
                    payload: payload,
                    at: new Date().getTime(),
                    status: 'SUBMIT',
                    observer: observer,
                    lambda: lambda,
                    queue: queue,
                    version: _this.appConfig.get('core', false).version
                };
                Api.startTimeoutCheck();
                _this.processTransaction(pid);
            });
        });
    };
    Api.prototype.transactionResponse = function (rx) {
        if (rx.pid) {
            console.log("API:CLIENT_RX : ", rx);
            var pid = rx.pid;
            var transaction = Api.transactions[pid];
            if (transaction) {
                var observer = transaction.observer;
                //CALC PERCENTAGES FROM CHUNKS
                var chunkId = rx.response.chunkId + 1;
                var chunkTotal = rx.response.chunkTotal;
                var progress = (100 / chunkTotal) * chunkId;
                if (!rx.response.data)
                    rx.response.data = [];
                if (Array.isArray(rx.response.data)) {
                    if (rx.response.data.length == 1) {
                        rx.response.data = rx.response.data[0];
                    }
                }
                if (rx.error) {
                    observer.error({ progress: progress, data: rx.response.data });
                }
                else {
                    observer.next({ progress: progress, data: rx.response.data });
                }
                observer.complete();
            }
            else {
                console.warn("TRANSACTION NOT FOUND FOR " + pid, rx);
            }
        }
        else
            return console.error("SOCKET:TX_RESPONSE INVALID : ", rx);
    };
    Api.prototype.processTransaction = function (pid) {
        console.log("PROCESS TRANSACTION : " + pid);
        var that = this;
        var transaction = Api.transactions[pid];
        var observer = transaction.observer;
        var queue = transaction.queue;
        if (queue) {
            //TODO : IMPLEMENT QUEUING
        }
        else {
            //MAKE THE CALL; 
            transaction.latency.deviceRequestAt = new Date().getTime();
            this.socketService.transaction(transaction);
        }
    };
    Api.checkTimeouts = function () {
        if (Object.keys(Api.transactions).length == 0) {
            Api.stopTimeoutCheck();
        }
        console.debug("CHECKING TIMEOUTS");
        for (var pid in Api.transactions) {
            var tx = Api.transactions[pid];
            var now = new Date().getTime();
            console.debug("IS " + now + " - " + tx.at + " > " + tx.timeout);
            if ((now - tx.at) > tx.timeout) {
                console.debug("TX FOR PID " + pid + " TIMED OUT");
                tx.observer.error({ progress: 0, data: "Transaction Timed Out" });
                delete Api.transactions[pid];
            }
        }
    };
    Api.startTimeoutCheck = function () {
        if (!Api.timeoutCheckInterval) {
            Api.timeoutCheckInterval = setInterval(Api.checkTimeouts, 1000);
        }
    };
    Api.stopTimeoutCheck = function () {
        if (Api.timeoutCheckInterval)
            clearInterval(Api.timeoutCheckInterval);
        Api.timeoutCheckInterval = false;
    };
    Api.transactions = null;
    Api.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Api.ctorParameters = function () { return [
        { type: Config, },
        { type: Socket, },
        { type: Payload, },
        { type: Events, },
    ]; };
    return Api;
}());
export { Api };
//# sourceMappingURL=index.js.map