import { Injectable } from '@angular/core';
import { Consul } from '../consul';
import { Storage } from '@ionic/storage';
import { Events } from 'ionic-angular';
var Config = (function () {
    function Config(storage, consul, events) {
        this.storage = storage;
        this.consul = consul;
        this.events = events;
        this.storeName = 'atajo.app.config';
        this.events.subscribe('config:change', function (key, value, time) {
            console.debug("atajo.config : GOT CONFIG CHANGE @ " + time + " EVENT FOR " + key + " = " + value);
            if (key == 'domain' && value != '') {
                // this.refresh().then().catch();
            }
        });
    }
    Config.prototype.initialize = function (domain) {
        var _this = this;
        if (domain === void 0) { domain = ""; }
        return new Promise(function (resolve, reject) {
            if (domain != "") {
                console.debug("atajo.config : Initializing with domain : " + domain);
                _this._set('domain', domain).then(function () {
                    _this.loadConfigFromStorage().then(function () {
                        _this.refresh().then(resolve).catch(reject);
                    });
                });
            }
            else {
                console.debug("atajo.config : Initializing");
                _this.loadConfigFromStorage().then(function () {
                    if (!_this.get('domain', false)) {
                        console.warn("atajo.config : Domain not configured. Please call YourConfigInstance.set('domain', 'yourRegisteredDomainName');");
                        reject();
                    }
                    else {
                        _this.refresh().then(resolve).catch(reject);
                    }
                });
            }
        });
    };
    Config.prototype.refresh = function (domain, reset) {
        var _this = this;
        if (domain === void 0) { domain = false; }
        if (reset === void 0) { reset = false; }
        console.debug("atajo.config : REFRESH");
        if (reset)
            this.consul.consulClientToken = null;
        return new Promise(function (resolve, reject) {
            _this.getRemoteConfig(domain || _this.get('domain', ''))
                .then(function (config) {
                _this.updateConfig(config);
                resolve();
            })
                .catch(function (error) {
                console.error(error);
                reject();
            });
        });
    };
    Config.prototype.set = function (key, value) {
        var conf = 'Config.STATE';
        try {
            value = (value !== null && typeof value === 'object') ? JSON.stringify(value) : ("'" + value + "'" || '');
            var cmd = conf + "." + key + " = " + value;
            console.debug("atajo.config : EVALUATING SET : " + cmd);
            var ret = eval(cmd);
            this.events.publish('config:change', key, value, Date.now());
            this.saveConfigToStorage().then().catch(function (error) { console.debug("atajo.config : SET ERROR : ", error); });
        }
        catch (e) {
            console.error("COULD NOT SET " + key + " == " + value + " => " + e.stack);
        }
    };
    Config.prototype._set = function (key, value) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var conf = 'Config.STATE';
            try {
                value = (value !== null && typeof value === 'object') ? JSON.stringify(value) : ("'" + value + "'" || '');
                var cmd = conf + "." + key + " = " + value;
                console.debug("atajo.config : EVALUATING SET : " + cmd);
                var ret = eval(cmd);
                _this.events.publish('config:change', key, value, Date.now());
                _this.saveConfigToStorage().then(resolve).catch(reject);
            }
            catch (e) {
                console.error("COULD NOT SET " + key + " == " + value + " => " + e.stack);
            }
        });
    };
    Config.prototype.get = function (value, defaultValue) {
        // console.debug("atajo.config : CONFIG IS : ", Config.STATE);
        if (typeof defaultValue == 'undefined') {
            defaultValue = false;
        }
        var conf = 'Config.STATE';
        try {
            var cmd = conf + '.' + value;
            console.debug("atajo.config : EVALUATING : " + cmd);
            var ret = eval(cmd);
            if (typeof ret == 'undefined' || ret == 'undefined') {
                return defaultValue;
            }
            else {
                //console.debug("atajo.config : APPCONFIG.GET (" + value + ") GOT : ", ret);
                var environment = Config.STATE.environment;
                //Config.STATE.domain = '';
                if (ret['dev'] && ret['qas'] && ret['prd']) {
                    console.debug("atajo.config : APPCONFIG.GET (" + value + ") FOUND ENV -> GOT (" + environment + ") : ", ret[environment]);
                    return ret[environment];
                }
                return ret;
            }
        }
        catch (e) {
            console.debug("atajo.config :COULD NOT GET " + value + " -> RETURNING DEFAULT OF : ", defaultValue);
            return defaultValue;
        }
    };
    Config.prototype._get = function (value, defaultValue) {
        var _this = this;
        console.debug("atajo.config : _get called");
        return new Promise(function (resolve) {
            _this.loadConfigFromStorage().then(function () {
                resolve(_this.get(value, defaultValue));
            });
        });
    };
    Config.prototype.getRemoteConfig = function (domain) {
        var _this = this;
        var that = this;
        domain = domain.toLowerCase();
        console.debug("atajo.config : GET REMOTE CONFIG FOR " + domain);
        return new Promise(function (resolve, reject) {
            if (!domain || domain == '') {
                reject("WILL NOT FETCH REMOTE CONFIG - DOMAIN NAME NOT SET");
                return;
            }
            var consulAPIUrl = _this.get('consulAPIUrl', false);
            var consulMapToken = _this.get('consulMapToken', false);
            _this.consul.getConfig(domain, consulAPIUrl, consulMapToken)
                .then(function (config) {
                console.debug("atajo.config : APP CONFIG GOT REMOTE CONFIG : ", config);
                if (!config) {
                    resolve(Config.STATE);
                }
                else if (config['notFound']) {
                    //this.toastService.toast('CONFIG NOT FOUND FOR : ' + domain.toUpperCase());
                    reject("CONSUL COULD NOT FIND CONFIG FOR: " + domain);
                }
                else {
                    console.debug("atajo.config : RESOLVING NEW CONFIG");
                    resolve(config);
                }
            });
        });
    };
    Config.prototype.updateConfig = function (config) {
        console.debug("atajo.config : UPDATING CONFIG WITH : ", config);
        for (var i in config) {
            this.set(i, config[i]);
        }
        console.debug("atajo.config : APP CONFIG NOW : ", Config.STATE);
    };
    Config.prototype.saveConfigToStorage = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var config = Config.STATE;
            console.debug("atajo.config : saveConfigToStorage : ", config);
            _this.storage.set(_this.storeName, JSON.stringify(config))
                .then(resolve)
                .catch(reject);
        });
    };
    Config.prototype.loadConfigFromStorage = function () {
        var _this = this;
        return new Promise(function (resolve) {
            _this.storage.ready().then(function () {
                _this.storage.get(_this.storeName)
                    .then(function (data) {
                    console.debug("atajo.config : getConfigFromStorage : ", data);
                    if (!data) {
                        console.debug("atajo.config : NO REMOTE CONFIG HAS BEEN LOADED");
                        resolve();
                        return;
                    }
                    try {
                        Config.STATE = (JSON.parse(data));
                        resolve();
                    }
                    catch (e) {
                        console.error(e);
                        resolve();
                    }
                }).catch(function (error) { console.error(error); });
            });
        });
    };
    Config.STATE = {
        domain: "",
        environment: "dev",
        consulAPIUrl: "https://consul.atajo.co.za/v1/kv/",
        consulMapToken: "cc110285-09b9-7767-dc1c-ab04fb24c784",
        update: {
            silent: false,
            key: {
                android: {
                    dev: "9lTd-2cbm6mDIz2BuqfNCctHV-aYc114e27d-b55f-4180-a8ed-945ecbc98085",
                    qas: "9lTd-2cbm6mDIz2BuqfNCctHV-aYc114e27d-b55f-4180-a8ed-945ecbc98085",
                    prd: "a6oJqbYO2wzndcqIpXDRdRCgWMB3c114e27d-b55f-4180-a8ed-945ecbc98085",
                },
                ios: {
                    dev: "JcPd4jtUseaYwwuBlK-rbqMWVCJ_c114e27d-b55f-4180-a8ed-945ecbc98085",
                    qas: "JcPd4jtUseaYwwuBlK-rbqMWVCJ_c114e27d-b55f-4180-a8ed-945ecbc98085",
                    prd: "uZKNOMhDtCVR4VFgwLBt3pjf9mV6c114e27d-b55f-4180-a8ed-945ecbc98085",
                }
            }
        },
        socket: {
            options: {
                reconnection: true,
                reconnectionAttempts: 50,
                reconnectionDelay: 1000,
                reconnectionDelayMax: 30000,
                randomizationFactor: 0.5,
                timeout: 10000,
                autoConnect: true
            },
            notify: true,
        }
    };
    Config.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Config.ctorParameters = function () { return [
        { type: Storage, },
        { type: Consul, },
        { type: Events, },
    ]; };
    return Config;
}());
export { Config };
//# sourceMappingURL=index.js.map