import { Injectable } from '@angular/core';
import { Http } from '../http';
var Consul = (function () {
    function Consul(http) {
        this.http = http;
    }
    Consul.prototype.getConfig = function (domain, consulAPIUrl, consulMapToken) {
        var _this = this;
        console.debug("CONSUL:GETCONFIG CALLED");
        this.consulAPIUrl = consulAPIUrl;
        this.consulMapToken = consulMapToken;
        return new Promise(function (resolve) {
            if (!_this.consulClientToken || _this.consulClientToken == null) {
                console.debug("atajo.consul : GETTING MAP FOR ", domain);
                _this.getClientToken(domain).then(function (token) {
                    if (!token) {
                        console.warn('atajo.consul : PRODUCT SUBDOMAIN (' + domain + ') NOT CONFIGURED ON CONSUL');
                        resolve({ notFound: true });
                        return;
                    }
                    console.debug("atajo.consul : CLIENT TOKEN IS ", token);
                    _this.consulClientToken = token.toString();
                    _this.getClientConfig(domain).then(function (config) {
                        console.log('config is : ', config);
                        resolve(config);
                    }).catch(function (error) { console.error(error); resolve(false); });
                }).catch(function (error) { console.error(error); resolve(false); });
            }
            else {
                console.debug("atajo.consul : CONFIG ALREADY FETCHED");
                resolve(false);
            }
        });
    };
    Consul.prototype.getClientToken = function (domain) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.http.get(_this.consulAPIUrl + 'map', { 'X-Consul-Token': _this.consulMapToken })
                .subscribe(function (data) {
                try {
                    var map = JSON.parse(window.atob(data[0].Value));
                    map[domain] ? resolve(map[domain]) : resolve(false);
                }
                catch (e) {
                    reject(e);
                }
            }, function (error) {
                reject(error);
            }, function () {
                resolve();
            });
        });
    };
    Consul.prototype.getClientConfig = function (domain) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            console.debug("atajo.consul : GETTING CLIENT CONFIG " + domain);
            _this.http.get(_this.consulAPIUrl + domain, { 'X-Consul-Token': _this.consulClientToken })
                .subscribe(function (data) {
                try {
                    var map = JSON.parse(window.atob(data[0].Value));
                    resolve(map);
                }
                catch (e) {
                    reject(e);
                }
            }, function (error) {
                reject(error);
            }, function () {
                resolve();
            });
        });
    };
    Consul.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Consul.ctorParameters = function () { return [
        { type: Http, },
    ]; };
    return Consul;
}());
export { Consul };
//# sourceMappingURL=index.js.map