import { Injectable } from '@angular/core';
import { Device } from '../device';
import { WebSql } from './websql';
import { Sqlite } from './sqlite';
var Database = (function () {
    function Database(device, websql, sqlite) {
        this.device = device;
        this.websql = websql;
        this.sqlite = sqlite;
        if (device.isCordova()) {
            sqlite.init();
        }
        else {
            websql.init();
        }
    }
    Database.prototype.create = function (table, columns) {
        if (this.device.isCordova()) {
            return this.sqlite.create(table, columns);
        }
        else {
            return this.websql.create(table, columns);
        }
    };
    Database.prototype.set = function (table, values) {
        if (this.device.isCordova()) {
            return this.sqlite.set(table, values);
        }
        else {
            return this.websql.set(table, values);
        }
    };
    Database.prototype.get = function (table, where) {
        if (this.device.isCordova()) {
            return this.sqlite.get(table, where);
        }
        else {
            return this.websql.get(table, where);
        }
    };
    Database.prototype.remove = function (table, where) {
        if (this.device.isCordova()) {
            return this.sqlite.remove(table, where);
        }
        else {
            return this.websql.remove(table, where);
        }
    };
    Database.prototype.nuke = function (table) {
        if (this.device.isCordova()) {
            return this.sqlite.nuke(table);
        }
        else {
            return this.websql.nuke(table);
        }
    };
    Database.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Database.ctorParameters = function () { return [
        { type: Device, },
        { type: WebSql, },
        { type: Sqlite, },
    ]; };
    return Database;
}());
export { Database };
//# sourceMappingURL=index.js.map