import { Injectable } from '@angular/core';
import { SQLite } from '@ionic-native/sqlite';
var Sqlite = (function () {
    function Sqlite(sqlite) {
        this.sqlite = sqlite;
    }
    Sqlite.prototype.init = function () {
        var _this = this;
        this.sqlite.create({
            name: 'atajo.client.db',
            location: 'default'
        })
            .then(function (db) {
            _this.db = db;
        })
            .catch(function (error) { return console.log(error); });
    };
    Sqlite.prototype.create = function (table, columns) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var columnsWithTypes = [];
            columns.forEach(function (column) {
                columnsWithTypes.push(column.name + ' ' + column.type);
            });
            _this.db.executeSql('CREATE TABLE IF NOT EXISTS ' + table + ' (' + columnsWithTypes.join(', ') + ')', {})
                .then(function () {
                resolve();
            })
                .catch(function (error) {
                console.error(error);
                reject();
            });
        });
    };
    Sqlite.prototype.set = function (table, values) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var promises = [];
            values.forEach(function (value) {
                var rowColumns = [];
                var rowValues = [];
                var valuePlaceholders = [];
                for (var v in value) {
                    if (value.hasOwnProperty(v)) {
                        rowColumns.push(v);
                        rowValues.push(JSON.stringify(value[v]));
                    }
                }
                rowColumns.forEach(function () {
                    valuePlaceholders.push('?');
                });
                promises.push(_this.db.executeSql('INSERT INTO ' + table + ' (' + rowColumns.join(', ') + ') VALUES (' + valuePlaceholders.join(', ') + ')', rowValues));
            });
            Promise.all(promises).then(function () {
                resolve();
            })
                .catch(function (error) {
                console.error(error);
                reject();
            });
        });
    };
    Sqlite.prototype.get = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
            _this.db.executeSql('SELECT * FROM ' + table + ' ' + whereStatement, {})
                .then(function (data) {
                var rows = [];
                var len = data.rows.length;
                for (var i = 0; i < len; ++i) {
                    var obj = data.rows.item(i);
                    var processedRow = {};
                    for (var r in obj) {
                        if (obj.hasOwnProperty(r)) {
                            processedRow[r] = JSON.parse(obj[r]);
                        }
                    }
                    rows.push(processedRow);
                }
                resolve(rows);
            })
                .catch(function (error) {
                console.error(error);
                reject();
            });
        });
    };
    Sqlite.prototype.remove = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
            _this.db.executeSql('DELETE FROM ' + table + ' ' + whereStatement, {})
                .then(function () {
                resolve();
            })
                .catch(function (error) {
                console.error(error);
                reject();
            });
        });
    };
    Sqlite.prototype.nuke = function (table) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.executeSql('DROP TABLE IF EXISTS ' + table, {})
                .then(function () {
                resolve();
            })
                .catch(function (error) {
                console.error(error);
                reject();
            });
        });
    };
    Sqlite.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Sqlite.ctorParameters = function () { return [
        { type: SQLite, },
    ]; };
    return Sqlite;
}());
export { Sqlite };
//# sourceMappingURL=sqlite.js.map