import { Injectable } from '@angular/core';
var WebSql = (function () {
    function WebSql() {
    }
    WebSql.prototype.init = function () {
        this.db = window.openDatabase("atajo.client", "1.0", "Atajo Client Database", 1000000);
    };
    WebSql.prototype.create = function (table, columns) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var columnsWithTypes = [];
                columns.forEach(function (column) {
                    columnsWithTypes.push(column.name + ' ' + column.type);
                });
                tx.executeSql('CREATE TABLE IF NOT EXISTS ' + table + ' (' + columnsWithTypes.join(', ') + ')');
            }, function (error) {
                console.error(error);
                reject();
            }, function () {
                resolve();
            });
        });
    };
    WebSql.prototype.set = function (table, values) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                values.forEach(function (value) {
                    var rowColumns = [];
                    var rowValues = [];
                    var valuePlaceholders = [];
                    for (var v in value) {
                        if (value.hasOwnProperty(v)) {
                            rowColumns.push(v);
                            rowValues.push(JSON.stringify(value[v]));
                        }
                    }
                    rowColumns.forEach(function () {
                        valuePlaceholders.push('?');
                    });
                    tx.executeSql('INSERT INTO ' + table + ' (' + rowColumns.join(', ') + ') VALUES (' + valuePlaceholders.join(', ') + ')', rowValues);
                });
            }, function (error) {
                console.error(error);
                reject();
            }, function () {
                resolve();
            });
        });
    };
    WebSql.prototype.get = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
                tx.executeSql('SELECT * FROM ' + table + ' ' + whereStatement, [], function (tx, rs) {
                    var rows = [];
                    var len = rs.rows.length;
                    for (var i = 0; i < len; ++i) {
                        var obj = rs.rows.item(i);
                        var processedRow = {};
                        for (var r in obj) {
                            if (obj.hasOwnProperty(r)) {
                                processedRow[r] = JSON.parse(obj[r]);
                            }
                        }
                        rows.push(processedRow);
                    }
                    resolve(rows);
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    WebSql.prototype.remove = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
                tx.executeSql('DELETE FROM ' + table + ' ' + whereStatement, [], function (tx, rs) {
                    resolve();
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    WebSql.prototype.nuke = function (table) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                tx.executeSql('DROP TABLE IF EXISTS ' + table, [], function (tx, rs) {
                    resolve();
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    WebSql.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    WebSql.ctorParameters = function () { return []; };
    return WebSql;
}());
export { WebSql };
//# sourceMappingURL=websql.js.map