var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Injectable } from '@angular/core';
import { Geolocation } from '@ionic-native/geolocation';
import { Device } from '../device';
import { BackgroundGeolocation } from '@ionic-native/background-geolocation';
import { AlertController } from 'ionic-angular';
import { Diagnostic } from '@ionic-native/diagnostic';
var GeolocationMock = (function (_super) {
    __extends(GeolocationMock, _super);
    function GeolocationMock() {
        return _super.call(this) || this;
    }
    GeolocationMock.prototype.getCurrentPosition = function () {
        return new Promise(function (resolve) {
            var options = {
                enableHighAccuracy: true,
                timeout: 2000,
                maximumAge: 3600000
            };
            navigator.geolocation.getCurrentPosition(function (position) {
                resolve(position);
            }, function (error) {
                console.warn("COULD NOT GET LOCATION : ", error);
                resolve({ timestamp: new Date().getTime(), coords: { latitude: 0, longitude: 0, accuracy: 0, altitudeAccuracy: 0, heading: 0, speed: 0 } });
            }, options);
        });
    };
    return GeolocationMock;
}(Geolocation));
var Location = (function () {
    function Location(device, backgroundGeolocation, alertCtrl, diagnostic) {
        this.device = device;
        this.backgroundGeolocation = backgroundGeolocation;
        this.alertCtrl = alertCtrl;
        this.diagnostic = diagnostic;
        this.geolocation = (document.URL.includes('https://') || document.URL.includes('http://')) ? new GeolocationMock() : new Geolocation();
        this.options = {
            enableHighAccuracy: true,
            timeout: 5000,
            maximumAge: 60000
        };
        this.checkLocationAuthorized();
    }
    Location.prototype.locationEnablePrompt = function () {
        var _this = this;
        this.alertCtrl.create({
            title: 'Location Permission',
            subTitle: 'This app requires location enabled. Tap OK to open Location Settings',
            buttons: [
                {
                    text: 'OK',
                    role: 'OK',
                    handler: function () {
                        _this.backgroundGeolocation.showLocationSettings();
                    }
                }
            ]
        }).present();
    };
    Location.prototype.locationRuntimePermission = function () {
        return this.diagnostic.requestRuntimePermission(this.diagnostic.permission.ACCESS_FINE_LOCATION);
        //cordova.plugins.diagnostic.permission.ACCESS_FINE_LOCATION
    };
    Location.prototype.checkLocationAuthorized = function () {
        var _this = this;
        if (this.device.isCordova()) {
            console.debug("CHECKING IF LOCATION IS ENABLED : ");
            this.diagnostic.isLocationAuthorized().then(function () {
                console.info("LOCATION IS AUTHORIZED");
            }).catch(function () {
                _this.diagnostic.requestLocationAuthorization().then(function () {
                    console.info("LOCATION AUTHORIZATION GRANTED");
                }).catch(function () {
                    console.warn("LOCATION AUTHORIZATION REJECTED");
                });
            });
            /*backgroundGeolocation.isLocationEnabled(function (enabled) {
              if (enabled) {
                backgroundGeolocation.start(
                  function () {
                    // service started successfully
                    // you should adjust your app UI for example change switch element to indicate
                    // that service is running
                  },
                  function (error) {
                    // Tracking has not started because of error
                    // you should adjust your app UI for example change switch element to indicate
                    // that service is not running
                    if (error.code === 2) {
                      if (window.confirm('Not authorized for location updates. Would you like to open app settings?')) {
                        backgroundGeolocation.showAppSettings();
                      }
                    } else {
                      window.alert('Start failed: ' + error.message);
                    }
                  }
                );
              } else {
                // Location services are disabled
                if (window.confirm('Location is disabled. Would you like to open location settings?')) {
                  backgroundGeolocation.showLocationSettings();
                }
              }
            });*/
        }
    };
    Location.prototype.getLocation = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.geolocation.getCurrentPosition(_this.options).then(resolve).catch(function (error) {
                console.error("LOCATION LOOKUP ERROR: ", error);
                _this.locationRuntimePermission().then(function (status) {
                    console.log("REQUESTED PERMISSION " + status);
                    _this.geolocation.getCurrentPosition().then(resolve);
                }).catch(function (error) {
                    console.error("REQUESTED PERMISSION " + error);
                });
            });
        });
    };
    /*
  
    backgroundGeolocation.configure(callbackFn, failureFn, {
      desiredAccuracy: 10,
      stationaryRadius: 20,
      distanceFilter: 30,
      url: 'http://192.168.81.15:3000/locations',
      httpHeaders: { 'X-FOO': 'bar' },
      maxLocations: 1000,
      // Android only section
      locationProvider: backgroundGeolocation.provider.ANDROID_ACTIVITY_PROVIDER,
      interval: 60000,
      fastestInterval: 5000,
      activitiesInterval: 10000,
      notificationTitle: 'Background tracking',
      notificationText: 'enabled',
      notificationIconColor: '#FEDD1E',
      notificationIconLarge: 'mappointer_large',
      notificationIconSmall: 'mappointer_small'
  });
  
  backgroundGeolocation.watchLocationMode(
    function (enabled) {
      if (enabled) {
        // location service are now enabled
        // call backgroundGeolocation.start
        // only if user already has expressed intent to start service
      } else {
        // location service are now disabled or we don't have permission
        // time to change UI to reflect that
      }
    },
    function (error) {
      console.log('Error watching location mode. Error:' + error);
    }
  );
  
  backgroundGeolocation.isLocationEnabled(function (enabled) {
    if (enabled) {
      backgroundGeolocation.start(
        function () {
          // service started successfully
          // you should adjust your app UI for example change switch element to indicate
          // that service is running
        },
        function (error) {
          // Tracking has not started because of error
          // you should adjust your app UI for example change switch element to indicate
          // that service is not running
          if (error.code === 2) {
            if (window.confirm('Not authorized for location updates. Would you like to open app settings?')) {
              backgroundGeolocation.showAppSettings();
            }
          } else {
            window.alert('Start failed: ' + error.message);
          }
        }
      );
    } else {
      // Location services are disabled
      if (window.confirm('Location is disabled. Would you like to open location settings?')) {
        backgroundGeolocation.showLocationSettings();
      }
    }
  });
  
    
  
  let watch = this.geolocation.watchPosition();
  watch.subscribe((data) => {
   // data can be a set of coordinates, or an error (if an error occurred).
   // data.coords.latitude
   // data.coords.longitude
  });
    */
    Location.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Location.ctorParameters = function () { return [
        { type: Device, },
        { type: BackgroundGeolocation, },
        { type: AlertController, },
        { type: Diagnostic, },
    ]; };
    return Location;
}());
export { Location };
//# sourceMappingURL=index.js.map