import { Injectable } from "@angular/core";
import io from "socket.io-client";
import { Events } from 'ionic-angular';
import { Config } from '../config';
import { Device } from '../device';
var Socket = (function () {
    function Socket(deviceProvider, events, config) {
        this.events = events;
        this.config = config;
        this.deviceProvider = deviceProvider;
        this.eventRegister = [];
    }
    Socket.prototype.connected = function () {
        return this.socket.connected;
    };
    Socket.prototype.disconnect = function () {
        this.socket.disconnect();
    };
    Socket.prototype.getSocket = function () {
        return this.socket;
    };
    Socket.prototype.connect = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var that = _this;
            Promise.all([
                _this.deviceProvider.getDevice(),
                _this.config.initialize()
            ]).then(function (config) {
                _this.domain = _this.config.get('domain', false);
                _this.channel = _this.config.get('core', false);
                _this.key = _this.config.get('key', false);
                _this.device = config[0];
                var socketOptions = _this.getSocketOptions();
                var socketEndpoint = _this.channel.protocol + '://' + _this.channel.host + ':' + _this.channel.port;
                console.debug("atajo.socket : SOCKET CONNECTING WITH KEY " + _this.key + " TO " + socketEndpoint + " - ", socketOptions);
                if (_this.socket) {
                    _this.socket.destroy();
                    delete _this.socket;
                }
                _this.socket = io(socketEndpoint, socketOptions);
                _this.addEvent('disconnect', function (data) {
                    console.warn("SOCKET DISCONNECTED : ", data);
                });
                _this.addEvent('connect', function (data) {
                    that.socket.emit('provider:status', { domain: _this.domain });
                });
                _this.addEvent('transaction', function (data) { });
                _this.addEvent('provider:connect', function (data) {
                    console.debug("atajo.socket :PROVIDER:CONNECT", data);
                });
                _this.addEvent('provider:disconnect', function (data) {
                    console.debug("atajo.socket :PROVIDER:DISCONNECT", data);
                });
                _this.addEvent('provider:status', function (data) {
                    console.debug("atajo.socket :PROVIDER:STATUS", data);
                });
                _this.addEvent('client:rx', function (data) {
                    console.debug("atajo.socket :CLIENT:RX", data);
                });
                resolve();
            }, function () {
                reject();
            });
        });
    };
    Socket.prototype.transaction = function (transaction) {
        var tx = {
            version: transaction.version,
            lambda: transaction.lambda,
            domain: this.config.get('domain', false),
            environment: this.config.get('environment', false),
            payload: transaction.payload,
            latency: transaction.latency
        };
        console.debug("atajo.socket : CLIENT:TX", tx);
        this.socket.emit('client:tx', tx);
    };
    Socket.prototype.addEvent = function (eventName, execute) {
        var _this = this;
        console.debug("atajo.socket :ADDING EVENT " + eventName + " TO SOCKETS");
        if (!this.socket) {
            console.error("SOCKET NOT INITIALIZED YET - CANNOT ADD EVENT");
            return;
        }
        this.eventRegister.push(eventName);
        this.socket.on(eventName, function (data) {
            console.debug("atajo.socket :GOT SOCKET EVENT : " + eventName + " => PUBLISHING TO " + 'socket:' + eventName);
            data = execute(data) || data;
            _this.events.publish('socket:' + eventName, data);
        });
    };
    Socket.prototype.removeEvent = function (eventName) {
        this.socket.removeListener(eventName);
        var index = this.eventRegister.indexOf(eventName, 0);
        if (index > -1) {
            this.eventRegister.splice(index, 1);
        }
        return this.eventRegister;
    };
    Socket.prototype.listEvents = function () {
        return this.eventRegister;
    };
    Socket.prototype.getSocketOptions = function () {
        var options = this.config.get('socket.options', {});
        options.query = {
            uuid: this.device.uuid,
            key: this.key,
            domain: this.domain,
            device: JSON.stringify(this.device)
        };
        return options;
    };
    Socket.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Socket.ctorParameters = function () { return [
        { type: Device, },
        { type: Events, },
        { type: Config, },
    ]; };
    return Socket;
}());
export { Socket };
//# sourceMappingURL=index.js.map