import { Injectable } from '@angular/core';
import { CodePush, SyncStatus } from '@ionic-native/code-push';
import { Config } from '../config';
import { Device } from '../device';
import { AlertController } from 'ionic-angular';
import { Utils } from '../utils';
import { Toast } from '../toast';
var Update = (function () {
    function Update(codePush, deviceProvider, toastService, alertCtrl, utilsProvider, appConfig) {
        var _this = this;
        this.codePush = codePush;
        this.deviceProvider = deviceProvider;
        this.toastService = toastService;
        this.alertCtrl = alertCtrl;
        this.utilsProvider = utilsProvider;
        this.appConfig = appConfig;
        console.log("PLATFORM IS : " + deviceProvider.getPlatformFromAgent());
        this.codePush.getCurrentPackage().then(function (details) {
            console.debug("CURRENT PACKAGE DETAILS : ", details);
            _this.currentPackage = details;
        }).catch(function (error) {
            console.error("ERROR FETCHING CURRENT PACKAGE DETAILS");
        });
        if (['ios', 'android', 'windows'].indexOf(deviceProvider.getPlatformFromAgent()) > -1) {
            this.deploymentKey = this.appConfig.get('update.key.' + deviceProvider.getPlatformFromAgent(), false);
            this.enabled = true;
            this.isAlertShowing = false;
            this.getCurrent().then(function (current) {
                console.log("CURRENT IS : ", current);
            });
        }
        else {
            this.updateNotAvailable();
            this.enabled = false;
        }
    }
    Update.prototype.getCurrent = function () {
        if (!this.enabled) {
            this.updateNotAvailable();
            return new Promise(function (resolve, reject) { });
        }
        return this.codePush.getCurrentPackage();
    };
    Update.prototype.checkForUpdate = function () {
        var _this = this;
        if (!this.enabled) {
            this.updateNotAvailable();
            return;
        }
        console.log("CODE PUSH CHECKING FOR UPDATES WITH KEY : " + this.deploymentKey);
        this.codePush.checkForUpdate(this.deploymentKey).then(function (updateStatus) {
            if (updateStatus === null) {
                console.log("CODE PUSH : APP IS UP TO DATE");
                return;
            }
            else {
                if (_this.appConfig.get('update.silent', false)) {
                    _this.sync();
                }
                else {
                    _this.confirmUpdate(updateStatus);
                }
            }
            console.log("UPDATE PACKAGE STATUS IS : ", updateStatus);
        });
    };
    Update.prototype.sync = function () {
        var _this = this;
        if (!this.enabled) {
            this.updateNotAvailable();
            return;
        }
        console.log("CODE PUSH CHECKING FOR UPDATES WITH KEY : " + this.deploymentKey);
        this.codePush.sync({ deploymentKey: this.deploymentKey }).subscribe(function (syncStatus) {
            console.log('Sync Status: ', syncStatus);
            var message = "";
            switch (syncStatus) {
                case SyncStatus.UP_TO_DATE:
                    message = 'Your app is up to date!';
                    break;
                case SyncStatus.IN_PROGRESS:
                    message = 'An update is in progress ..';
                    break;
                case SyncStatus.CHECKING_FOR_UPDATE:
                    message = 'Checking for update ..';
                    break;
                case SyncStatus.DOWNLOADING_PACKAGE:
                    _this.toastService.toast('Downloading package ..');
                    break;
                case SyncStatus.INSTALLING_UPDATE:
                    _this.toastService.toast('Installing update ..');
                    break;
                case SyncStatus.UPDATE_INSTALLED:
                    _this.confirmRestart();
                    break;
                case SyncStatus.ERROR:
                    _this.toastService.toast('An update error occurred :( ...');
                    break;
                default:
                    message = 'An unhandled sync status ..';
                    break;
            }
            console.log("UPDATE STATUS MESSAGE : " + message);
        });
    };
    Update.prototype.restart = function () {
        this.codePush.restartApplication();
    };
    Update.prototype.confirmRestart = function () {
        if (this.isAlertShowing) {
            return;
        }
        this.isAlertShowing = true;
        var that = this;
        var alert = this.alertCtrl.create({
            title: 'Update Downloaded',
            message: 'A new update was installed and will be available on next app restart. <br><br> Would you like to restart the app now?',
            buttons: [
                {
                    text: 'NO',
                    role: 'cancel',
                    handler: function () { that.isAlertShowing = false; console.log('Restart Cancelled'); }
                },
                {
                    text: 'YES',
                    handler: function () { that.isAlertShowing = false; console.log('Restart Accepted'); that.restart(); }
                }
            ]
        });
        alert.present();
    };
    Update.prototype.confirmUpdate = function (details) {
        if (this.isAlertShowing) {
            return;
        }
        this.isAlertShowing = true;
        var that = this;
        var alert = this.alertCtrl.create({
            title: 'Update Version ' + details.appVersion,
            message: 'BUILD ' + details.packageHash.substring(0, 8) + ' (' + this.utilsProvider.bytesToSize(details.packageSize) + ')<br><br>' + details.description,
            buttons: [
                {
                    text: 'No Thanks',
                    role: 'cancel',
                    handler: function () { that.isAlertShowing = false; console.log('Update Cancelled'); }
                },
                {
                    text: 'UPDATE',
                    handler: function () { that.isAlertShowing = false; console.log('Update Accepted'); that.sync(); }
                }
            ]
        });
        alert.present();
    };
    Update.prototype.isSuccessful = function () {
        if (!this.enabled) {
            this.updateNotAvailable();
            return;
        }
        this.codePush.notifyApplicationReady().then(function (message) {
            console.log("UPDATE APPLIED", message);
        }).catch(function (error) { console.error("CODE PUSH notifyAppReady ERROR : ", error); });
    };
    Update.prototype.updateNotAvailable = function () {
        console.warn('OTA UPDATES NOT AVAILABLE ON THIS DEVICE');
    };
    Update.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    Update.ctorParameters = function () { return [
        { type: CodePush, },
        { type: Device, },
        { type: Toast, },
        { type: AlertController, },
        { type: Utils, },
        { type: Config, },
    ]; };
    return Update;
}());
export { Update };
//# sourceMappingURL=index.js.map