import { Injectable } from '@angular/core';
import { Consul } from '../consul';
import { Storage } from '@ionic/storage';
import { Events } from 'ionic-angular';

@Injectable()
export class Config {

    private storeName: string;

    constructor(private storage: Storage, private consul: Consul, private events: Events) {

        this.storeName = 'atajo.app.config';
        this.events.subscribe('config:change', (key, value, time) => {
            console.debug("atajo.config : GOT CONFIG CHANGE @ " + time + " EVENT FOR " + key + " = " + value);
            if (key == 'domain' && value != '') {
                // this.refresh().then().catch();
            }
        });

    }

    private static STATE = {

        domain: "",
        environment: "dev",
        consulAPIUrl: "https://consul.atajo.co.za/v1/kv/",
        consulMapToken: "cc110285-09b9-7767-dc1c-ab04fb24c784",

        update: {

            silent: false, //Automatically check for and update app on next restart (true) || Notify user update is available and prompt to update (false)
            key: {
                android: {
                    dev: "9lTd-2cbm6mDIz2BuqfNCctHV-aYc114e27d-b55f-4180-a8ed-945ecbc98085",
                    qas: "9lTd-2cbm6mDIz2BuqfNCctHV-aYc114e27d-b55f-4180-a8ed-945ecbc98085",
                    prd: "a6oJqbYO2wzndcqIpXDRdRCgWMB3c114e27d-b55f-4180-a8ed-945ecbc98085",
                },

                ios: {
                    dev: "JcPd4jtUseaYwwuBlK-rbqMWVCJ_c114e27d-b55f-4180-a8ed-945ecbc98085",
                    qas: "JcPd4jtUseaYwwuBlK-rbqMWVCJ_c114e27d-b55f-4180-a8ed-945ecbc98085",
                    prd: "uZKNOMhDtCVR4VFgwLBt3pjf9mV6c114e27d-b55f-4180-a8ed-945ecbc98085",
                }
            }



        },

        socket: {
            options: {
                reconnection: true,
                reconnectionAttempts: 50,
                reconnectionDelay: 1000,
                reconnectionDelayMax: 30000,
                randomizationFactor: 0.5,
                timeout: 10000,
                autoConnect: true
            },
            notify: true, //show a toast when socket connection status changes

        }
    }


    public initialize(domain: string = ""): Promise<null> {



        return new Promise((resolve, reject) => {


            if (domain != "") {
                console.debug("atajo.config : Initializing with domain : " + domain);
                this._set('domain', domain).then(() => {

                    this.loadConfigFromStorage().then(() => {
                        this.refresh().then(resolve).catch(reject);
                    });
                });

            } else {

                console.debug("atajo.config : Initializing");
                this.loadConfigFromStorage().then(() => {

                    if (!this.get('domain', false)) {
                        console.warn("atajo.config : Domain not configured. Please call YourConfigInstance.set('domain', 'yourRegisteredDomainName');");
                        reject();
                    } else {
                        this.refresh().then(resolve).catch(reject);

                    }


                });
            }

        })


    }

    public refresh(domain = false, reset = false) {

        console.debug("atajo.config : REFRESH")

        if (reset) this.consul.consulClientToken = null;

        return new Promise((resolve, reject) => {

            this.getRemoteConfig(domain || this.get('domain', ''))
                .then(config => {

                    this.updateConfig(config);
                    resolve();

                })
                .catch(error => {

                    console.error(error);
                    reject();

                });



        })


    }

    public set(key: string, value: any) {

        let conf = 'Config.STATE';


        try {
            value = (value !== null && typeof value === 'object') ? JSON.stringify(value) : ("'" + value + "'" || '');
            var cmd = conf + "." + key + " = " + value;
            console.debug("atajo.config : EVALUATING SET : " + cmd);
            var ret = eval(cmd);
            this.events.publish('config:change', key, value, Date.now());
            this.saveConfigToStorage().then().catch(error => { console.debug("atajo.config : SET ERROR : ", error) })


        } catch (e) {
            console.error("COULD NOT SET " + key + " == " + value + " => " + e.stack);
        }

    }

    public _set(key: string, value: any) {

        return new Promise((resolve, reject) => {

            let conf = 'Config.STATE';

            try {
                value = (value !== null && typeof value === 'object') ? JSON.stringify(value) : ("'" + value + "'" || '');
                var cmd = conf + "." + key + " = " + value;
                console.debug("atajo.config : EVALUATING SET : " + cmd);
                var ret = eval(cmd);
                this.events.publish('config:change', key, value, Date.now());
                this.saveConfigToStorage().then(resolve).catch(reject);


            } catch (e) {
                console.error("COULD NOT SET " + key + " == " + value + " => " + e.stack);
            }


        })

    }



    public get(value: string, defaultValue: any) {

        // console.debug("atajo.config : CONFIG IS : ", Config.STATE);

        if (typeof defaultValue == 'undefined') {
            defaultValue = false;
        }

        var conf = 'Config.STATE';

        try {
            var cmd = conf + '.' + value;
            console.debug("atajo.config : EVALUATING : " + cmd);
            var ret = eval(cmd);
            if (typeof ret == 'undefined' || ret == 'undefined') {
                return defaultValue;
            } else {

                //console.debug("atajo.config : APPCONFIG.GET (" + value + ") GOT : ", ret);
                const environment = Config.STATE.environment;

                //Config.STATE.domain = '';

                if (ret['dev'] && ret['qas'] && ret['prd']) {
                    console.debug("atajo.config : APPCONFIG.GET (" + value + ") FOUND ENV -> GOT (" + environment + ") : ", ret[environment]);
                    return ret[environment];
                }

                return ret;
            }
        } catch (e) {
            console.debug("atajo.config :COULD NOT GET " + value + " -> RETURNING DEFAULT OF : ", defaultValue);
            return defaultValue;
        }


    }


    _get(value: string, defaultValue: any) {

        console.debug("atajo.config : _get called");

        return new Promise(resolve => {

            this.loadConfigFromStorage().then(() => {

                resolve(this.get(value, defaultValue));

            });


        })

    }


    public getRemoteConfig(domain: string) {

        var that = this;

        domain = domain.toLowerCase();

        console.debug("atajo.config : GET REMOTE CONFIG FOR " + domain);

        return new Promise((resolve, reject) => {

            if (!domain || domain == '') {
                reject("WILL NOT FETCH REMOTE CONFIG - DOMAIN NAME NOT SET");
                return;
            }

            const consulAPIUrl = this.get('consulAPIUrl', false);
            const consulMapToken = this.get('consulMapToken', false);

            this.consul.getConfig(domain, consulAPIUrl, consulMapToken)
                .then(config => {

                    console.debug("atajo.config : APP CONFIG GOT REMOTE CONFIG : ", config);

                    if (!config) {

                        resolve(Config.STATE);


                    } else if (config['notFound']) {
                        //this.toastService.toast('CONFIG NOT FOUND FOR : ' + domain.toUpperCase());
                        reject("CONSUL COULD NOT FIND CONFIG FOR: " + domain);


                    } else {
                        console.debug("atajo.config : RESOLVING NEW CONFIG")
                        resolve(config);
                    }


                })

        })

    }

    private updateConfig(config) {

        console.debug("atajo.config : UPDATING CONFIG WITH : ", config);

        for (var i in config) {
            this.set(i, config[i]);
        }

        console.debug("atajo.config : APP CONFIG NOW : ", Config.STATE);


    }

    private saveConfigToStorage() {

        return new Promise((resolve, reject) => {

            let config = Config.STATE;
            console.debug("atajo.config : saveConfigToStorage : ", config);
            this.storage.set(this.storeName, JSON.stringify(config))
                .then(resolve)
                .catch(reject);

        })


    }

    private loadConfigFromStorage() {

        return new Promise(resolve => {

            this.storage.ready().then(() => {
                this.storage.get(this.storeName)
                    .then(data => {

                        console.debug("atajo.config : getConfigFromStorage : ", data);

                        if (!data) {
                            console.debug("atajo.config : NO REMOTE CONFIG HAS BEEN LOADED");
                            resolve();
                            return;
                        }
                        try {
                            Config.STATE = (JSON.parse(data));
                            resolve();
                        } catch (e) {
                            console.error(e);
                            resolve();
                        }


                    }).catch(error => { console.error(error); })


            });



        })



    }


}