
import { Injectable } from '@angular/core';
import { Http } from '../http';
import { Observable } from 'rxjs/Observable';


@Injectable()
export class Consul {

    consulAPIUrl: string;
    consulMapToken: string;
    consulClientToken: string;

    constructor(private http: Http) { }

    getConfig(domain: string, consulAPIUrl, consulMapToken): Promise<object | boolean> {

        console.debug("CONSUL:GETCONFIG CALLED");

        this.consulAPIUrl = consulAPIUrl;
        this.consulMapToken = consulMapToken;

        return new Promise(resolve => {

            if (!this.consulClientToken || this.consulClientToken == null) {

                console.debug("atajo.consul : GETTING MAP FOR ", domain);
                this.getClientToken(domain).then(token => {

                    if (!token) {
                        console.warn('atajo.consul : PRODUCT SUBDOMAIN (' + domain + ') NOT CONFIGURED ON CONSUL');
                        resolve({ notFound: true });
                        return;
                    }


                    console.debug("atajo.consul : CLIENT TOKEN IS ", token);
                    this.consulClientToken = token.toString();
                    this.getClientConfig(domain).then(config => {

                        console.log('config is : ', config);
                        resolve(config);

                    }).catch(error => { console.error(error); resolve(false); })

                }).catch(error => { console.error(error); resolve(false); })


            }
            else {
                console.debug("atajo.consul : CONFIG ALREADY FETCHED");
                resolve(false);
            }

        });


    }

    getClientToken(domain: string) {

        return new Promise((resolve, reject) => {

            this.http.get(this.consulAPIUrl + 'map', { 'X-Consul-Token': this.consulMapToken })
                .subscribe(data => {

                    try {
                        const map: any = JSON.parse(window.atob(data[0].Value));
                        map[domain] ? resolve(map[domain]) : resolve(false);
                    }
                    catch (e) {
                        reject(e);
                    }

                }, error => {

                    reject(error);

                }, () => {

                    resolve();

                });



        })




    }

    getClientConfig(domain: string) {

        return new Promise((resolve, reject) => {

            console.debug("atajo.consul : GETTING CLIENT CONFIG " + domain);
            this.http.get(this.consulAPIUrl + domain, { 'X-Consul-Token': this.consulClientToken })
                .subscribe(data => {

                    try {
                        const map: any = JSON.parse(window.atob(data[0].Value));
                        resolve(map);
                    }
                    catch (e) {
                        reject(e);
                    }

                }, error => {

                    reject(error);

                }, () => {

                    resolve();

                });



        })




    }
}

