import {Injectable} from '@angular/core';
import {Device} from '../device';
import { WebSql } from './websql';
import { Sqlite } from './sqlite';

@Injectable()
export class Database {

  constructor(private device: Device, private websql: WebSql, private sqlite: Sqlite) {
    if (device.isCordova()) {
      sqlite.init();
    } else {
      websql.init();
    }
  }

  public create(table, columns) {
    if (this.device.isCordova()) {
      return this.sqlite.create(table, columns);
    } else {
      return this.websql.create(table, columns);
    }
  }

  public set(table, values) {
    if (this.device.isCordova()) {
      return this.sqlite.set(table, values);
    } else {
      return this.websql.set(table, values);
    }
  }

  public get(table, where?) {
    if (this.device.isCordova()) {
      return this.sqlite.get(table, where);
    } else {
      return this.websql.get(table, where);
    }
  }

  public remove(table, where?) {
    if (this.device.isCordova()) {
      return this.sqlite.remove(table, where);
    } else {
      return this.websql.remove(table, where);
    }
  }

  public nuke(table) {
    if (this.device.isCordova()) {
      return this.sqlite.nuke(table);
    } else {
      return this.websql.nuke(table);
    }
  }

}
