import {Injectable} from '@angular/core';
import { SQLite, SQLiteObject } from '@ionic-native/sqlite';

@Injectable()
export class Sqlite {

  private db;

  constructor(private sqlite: SQLite) {

  }

  public init() {
    this.sqlite.create({
      name: 'atajo.client.db',
      location: 'default'
    })
      .then((db: SQLiteObject) => {
        this.db = db;
      })
      .catch(error => console.log(error));
  }

  public create(table, columns) {
    return new Promise((resolve, reject) => {

      let columnsWithTypes = [];
      columns.forEach((column) => {
        columnsWithTypes.push(column.name + ' ' + column.type);
      });

      this.db.executeSql('CREATE TABLE IF NOT EXISTS ' + table + ' (' + columnsWithTypes.join(', ') + ')', {})
        .then(() => {
          resolve();
        })
        .catch(error => {
          console.error(error);
          reject();
        });

    });
  }

  public set(table, values) {
    return new Promise((resolve, reject) => {

      let promises = [];
      values.forEach((value) => {
        let rowColumns = [];
        let rowValues = [];
        let valuePlaceholders = [];
        for (let v in value) {
          if (value.hasOwnProperty(v)) {
            rowColumns.push(v);
            rowValues.push(JSON.stringify(value[v]));
          }
        }
        rowColumns.forEach(() => {
          valuePlaceholders.push('?');
        });
        promises.push(this.db.executeSql('INSERT INTO ' + table + ' (' + rowColumns.join(', ') + ') VALUES (' + valuePlaceholders.join(', ') + ')', rowValues));
      });

      Promise.all(promises).then(() => {
        resolve();
      })
      .catch(error => {
        console.error(error);
        reject();
      });

    });
  }

  public get(table, where?) {
    return new Promise((resolve, reject) => {
      let whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';

      this.db.executeSql('SELECT * FROM ' + table + ' ' + whereStatement, {})
        .then((data) => {

          let rows = [];
          let len = data.rows.length;
          for (let i = 0; i < len; ++i) {
            let obj = data.rows.item(i);
            let processedRow = {};
            for (let r in obj) {
              if (obj.hasOwnProperty(r)) {
                processedRow[r] = JSON.parse(obj[r]);
              }
            }
            rows.push(processedRow);
          }

          resolve(rows);
        })
        .catch(error => {
          console.error(error);
          reject();
        });
    });
  }

  public remove(table, where?) {
    return new Promise((resolve, reject) => {
      let whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
      this.db.executeSql('DELETE FROM ' + table + ' ' + whereStatement, {})
        .then(() => {
          resolve();
        })
        .catch(error => {
          console.error(error);
          reject();
        });
    });
  }

  public nuke(table) {
    return new Promise((resolve, reject) => {
      this.db.executeSql('DROP TABLE IF EXISTS ' + table, {})
        .then(() => {
          resolve();
        })
        .catch(error => {
          console.error(error);
          reject();
        });
    });
  }

}
