import {Injectable} from '@angular/core';

@Injectable()
export class WebSql {

  private db;

  constructor() {

  }

  public init() {
    this.db = (<any>window).openDatabase("atajo.client", "1.0", "Atajo Client Database", 1000000);
  }

  public create(table, columns) {
    return new Promise((resolve, reject) => {
      this.db.transaction(function (tx) {
        let columnsWithTypes = [];
        columns.forEach((column) => {
          columnsWithTypes.push(column.name + ' ' + column.type);
        });
        tx.executeSql('CREATE TABLE IF NOT EXISTS ' + table + ' (' + columnsWithTypes.join(', ') + ')');
      }, function (error) {
        console.error(error);
        reject();
      }, function () {
        resolve();
      });
    });
  }

  public set(table, values) {
    return new Promise((resolve, reject) => {
      this.db.transaction(function (tx) {
        values.forEach((value) => {
          let rowColumns = [];
          let rowValues = [];
          let valuePlaceholders = [];
          for (let v in value) {
            if (value.hasOwnProperty(v)) {
              rowColumns.push(v);
              rowValues.push(JSON.stringify(value[v]));
            }
          }
          rowColumns.forEach(() => {
            valuePlaceholders.push('?');
          });
          tx.executeSql('INSERT INTO ' + table + ' (' + rowColumns.join(', ') + ') VALUES (' + valuePlaceholders.join(', ') + ')', rowValues);
        });
      }, function (error) {
        console.error(error);
        reject();
      }, function () {
        resolve();
      });
    });
  }

  public get(table, where?) {
    return new Promise((resolve, reject) => {
      this.db.transaction(function (tx) {
        let whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
        tx.executeSql('SELECT * FROM ' + table + ' ' + whereStatement, [], function (tx, rs) {
          let rows = [];
          let len = rs.rows.length;
          for (let i = 0; i < len; ++i) {
            let obj = rs.rows.item(i);
            let processedRow = {};
            for (let r in obj) {
              if (obj.hasOwnProperty(r)) {
                processedRow[r] = JSON.parse(obj[r]);
              }
            }
            rows.push(processedRow);
          }
          resolve(rows);
        }, function (tx, error) {
          console.error(error);
          reject();
        });
      });
    });
  }

  public remove(table, where?) {
    return new Promise((resolve, reject) => {
      this.db.transaction(function (tx) {
        let whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
        tx.executeSql('DELETE FROM ' + table + ' ' + whereStatement, [], function (tx, rs) {
          resolve();
        }, function (tx, error) {
          console.error(error);
          reject();
        });
      });
    });
  }

  public nuke(table) {
    return new Promise((resolve, reject) => {
      this.db.transaction(function (tx) {
        tx.executeSql('DROP TABLE IF EXISTS ' + table, [], function (tx, rs) {
          resolve();
        }, function (tx, error) {
          console.error(error);
          reject();
        });
      });
    });
  }

}
