import { Injectable } from '@angular/core';
import { Geolocation, Geoposition } from '@ionic-native/geolocation';
import { Device } from '../device';
import { BackgroundGeolocation } from '@ionic-native/background-geolocation';
import { AlertController } from 'ionic-angular';
import { Diagnostic } from '@ionic-native/diagnostic';


class GeolocationMock extends Geolocation {

    constructor() {
        super();
    }

    getCurrentPosition(): Promise<any> {

        return new Promise(resolve => {

            var options = {
                enableHighAccuracy: true,
                timeout: 2000,
                maximumAge: 3600000
            };

            navigator.geolocation.getCurrentPosition(position => {

                resolve(position);

            }, error => {
                console.warn("COULD NOT GET LOCATION : ", error);
                resolve({ timestamp: new Date().getTime(), coords: { latitude: 0, longitude: 0, accuracy: 0, altitudeAccuracy: 0, heading: 0, speed: 0 } });
            }, options);

        })


    }

}



@Injectable()
export class Location {

    geolocation: any;
    options: any;


    constructor(private device: Device, private backgroundGeolocation: BackgroundGeolocation, private alertCtrl: AlertController, private diagnostic: Diagnostic) {

        this.geolocation = (document.URL.includes('https://') || document.URL.includes('http://')) ? new GeolocationMock() : new Geolocation();
        this.options = {
            enableHighAccuracy: true,
            timeout: 5000,
            maximumAge: 60000
        };

        this.checkLocationAuthorized();


    }

    locationEnablePrompt() {




        this.alertCtrl.create({
            title: 'Location Permission',
            subTitle: 'This app requires location enabled. Tap OK to open Location Settings',
            buttons: [
                {
                    text: 'OK',
                    role: 'OK',
                    handler: () => {

                        this.backgroundGeolocation.showLocationSettings();

                    }
                }
            ]
        }).present();


    }

    locationRuntimePermission() {

        return this.diagnostic.requestRuntimePermission(this.diagnostic.permission.ACCESS_FINE_LOCATION);
        //cordova.plugins.diagnostic.permission.ACCESS_FINE_LOCATION


    }

    private checkLocationAuthorized() {

        if (this.device.isCordova()) {

            console.debug("CHECKING IF LOCATION IS ENABLED : ");

            this.diagnostic.isLocationAuthorized().then(
                () => {

                    console.info("LOCATION IS AUTHORIZED");

                }
            ).catch(() => {

                this.diagnostic.requestLocationAuthorization().then(() => {

                    console.info("LOCATION AUTHORIZATION GRANTED");

                }).catch(() => {

                    console.warn("LOCATION AUTHORIZATION REJECTED");


                });


            })


            /*backgroundGeolocation.isLocationEnabled(function (enabled) {
              if (enabled) {
                backgroundGeolocation.start(
                  function () {
                    // service started successfully
                    // you should adjust your app UI for example change switch element to indicate
                    // that service is running
                  },
                  function (error) {
                    // Tracking has not started because of error
                    // you should adjust your app UI for example change switch element to indicate
                    // that service is not running
                    if (error.code === 2) {
                      if (window.confirm('Not authorized for location updates. Would you like to open app settings?')) {
                        backgroundGeolocation.showAppSettings();
                      }
                    } else {
                      window.alert('Start failed: ' + error.message);
                    }
                  }
                );
              } else {
                // Location services are disabled
                if (window.confirm('Location is disabled. Would you like to open location settings?')) {
                  backgroundGeolocation.showLocationSettings();
                }
              }
            });*/

        }


    }

    public getLocation() {

        return new Promise((resolve, reject) => {

            this.geolocation.getCurrentPosition(this.options).then(resolve).catch(error => {

                console.error("LOCATION LOOKUP ERROR: ", error);

                this.locationRuntimePermission().then(status => {

                    console.log("REQUESTED PERMISSION " + status)

                    this.geolocation.getCurrentPosition().then(resolve);



                }).catch(error => {

                    console.error("REQUESTED PERMISSION " + error);

                })


            });

        })


    }

    /*
  
    backgroundGeolocation.configure(callbackFn, failureFn, {
      desiredAccuracy: 10,
      stationaryRadius: 20,
      distanceFilter: 30,
      url: 'http://192.168.81.15:3000/locations',
      httpHeaders: { 'X-FOO': 'bar' },
      maxLocations: 1000,
      // Android only section
      locationProvider: backgroundGeolocation.provider.ANDROID_ACTIVITY_PROVIDER,
      interval: 60000,
      fastestInterval: 5000,
      activitiesInterval: 10000,
      notificationTitle: 'Background tracking',
      notificationText: 'enabled',
      notificationIconColor: '#FEDD1E',
      notificationIconLarge: 'mappointer_large',
      notificationIconSmall: 'mappointer_small'
  });
  
  backgroundGeolocation.watchLocationMode(
    function (enabled) {
      if (enabled) {
        // location service are now enabled
        // call backgroundGeolocation.start
        // only if user already has expressed intent to start service
      } else {
        // location service are now disabled or we don't have permission
        // time to change UI to reflect that
      }
    },
    function (error) {
      console.log('Error watching location mode. Error:' + error);
    }
  );
  
  backgroundGeolocation.isLocationEnabled(function (enabled) {
    if (enabled) {
      backgroundGeolocation.start(
        function () {
          // service started successfully
          // you should adjust your app UI for example change switch element to indicate
          // that service is running
        },
        function (error) {
          // Tracking has not started because of error
          // you should adjust your app UI for example change switch element to indicate
          // that service is not running
          if (error.code === 2) {
            if (window.confirm('Not authorized for location updates. Would you like to open app settings?')) {
              backgroundGeolocation.showAppSettings();
            }
          } else {
            window.alert('Start failed: ' + error.message);  
          }
        }
      );
    } else {
      // Location services are disabled
      if (window.confirm('Location is disabled. Would you like to open location settings?')) {
        backgroundGeolocation.showLocationSettings();
      }
    }
  });
  
    
  
  let watch = this.geolocation.watchPosition();
  watch.subscribe((data) => {
   // data can be a set of coordinates, or an error (if an error occurred).
   // data.coords.latitude
   // data.coords.longitude
  });
    */


}